from __future__ import annotations

from typing import Dict


"""
Map of string key names to string key values. This map is used to store option names and their corresponding option values.
"""
OptionKeyToOptionValueMap = Dict[str, str]
