from tcsoa.gen.Internal_CodeCoverage._2011_06.services import CodeCoverageManagementService as imp0
from tcsoa.base import TcService


class CodeCoverageManagementService(TcService):
    performOperation = imp0.performOperation
