from tcsoa.gen.Configurator._2014_12.services import ConfiguratorManagementService as imp0
from tcsoa.base import TcService


class ConfiguratorManagementService(TcService):
    getAdmissibility = imp0.getAdmissibility
